@@ -1,42 +1,48 @@
 #' Knit a document
-#' 
+#'
 #' This function takes an input file, extracts the R code in it according to a
 #' list of patterns, evaluates the code and writes the output in another file.
 #' It can also tangle R source code from the input document (\code{purl()} is a
 #' wrapper to \code{knit(..., tangle = TRUE)}).
-#' 
+#'
 #' For most of the time, it is not necessary to set any options outside the
-#' input document; in other words, a single call like 
+#' input document; in other words, a single call like
 #' \code{knit('my_input.Rnw')} is usually enough. This function will try to
 #' determine many internal settings automatically. For the sake of
 #' reproducibility, it is a better practice to include the options inside the
 #' input document (to be self-contained), instead of setting them before
 #' knitting the document.
-#' 
+#'
 #' First the filename of the output document is determined in this way:
-#' \file{foo.Rnw} generates \file{foo.tex}, and for other types of files, the
-#' file extension is reserved; if the filename contains \samp{_knit_}, this part
-#' will be removed in the output file, e.g., \file{foo_knit_.html} creates the
-#' output \file{foo.html}, so you can use files named in this way as templates;
-#' if \samp{_knit_} is not found in the filename, \file{foo.ext} will produce 
+#' \file{foo.Rnw} generates \file{foo.tex}, and other filename extensions like
+#' \file{.Rtex}, \file{.Rhtml} (\file{.Rhtm}) and \file{.Rmd}
+#' (\file{.Rmarkdown}) will generate \file{.tex}, \file{.html} and \file{.md}
+#' respectively. For other types of files, the file extension is reserved; if
+#' the filename contains \samp{_knit_}, this part will be removed in the output
+#' file, e.g., \file{foo_knit_.html} creates the output \file{foo.html}; if
+#' \samp{_knit_} is not found in the filename, \file{foo.ext} will produce
 #' \file{foo-out.ext}. If \code{tangle = TRUE}, \file{foo.ext} generates an R
 #' script \file{foo.R}.
-#' 
-#' Based on the file extension of the input document, a list of patterns will be
-#' used to extract R code in the document. All built-in pattern lists can be
-#' found in \code{opts_knit$get('all.patterns')} (call it \code{apat}).
-#' \samp{Rnw} files use the list \code{apat$rnw}, \samp{tex} uses the list
-#' \code{apat$tex}, \samp{brew} uses \code{apat$brew} and HTML-like files use
-#' \code{apat$html} (e.g. \samp{html} and \samp{md} files). You can manually set
-#' the pattern list using the \code{\link{knit_patterns}} object, and 
+#'
+#' We need a set of syntax to identify special markups for R code chunks and R
+#' options, etc. The syntax is defined in a pattern list. All built-in pattern
+#' lists can be found in \code{all_patterns} (call it \code{apat}). First the
+#' content of the input document is matched against all pattern lists to
+#' automatically which pattern list is being used. If automatic detection
+#' failed, the pattern list will be decided based on the filename extension of
+#' the input document. \samp{Rnw} files use the list \code{apat$rnw}, \samp{tex}
+#' uses the list \code{apat$tex}, \samp{brew} uses \code{apat$brew} and
+#' HTML-like files use \code{apat$html} (e.g. \samp{html} and \samp{md} files).
+#' You can manually set the pattern list using the \code{\link{knit_patterns}}
+#' object or the \code{\link{pat_rnw}} series functions in advance and
 #' \pkg{knitr} will respect the setting.
-#' 
+#'
 #' According to the output format (\code{opts_knit$get('out.format')}), a set of
-#' output hooks will be set to mark up results from R (see 
+#' output hooks will be set to mark up results from R (see
 #' \code{\link{render_latex}}). The output format can be LaTeX, Sweave and HTML,
 #' etc. The output hooks decide how to mark up the results (you can customize
 #' the hooks).
-#' 
+#'
 #' See the package website and manuals in the references to know more about
 #' \pkg{knitr}, including the full documentation of chunk options and demos,
 #' etc.
@@ -46,35 +52,36 @@
 #' @param tangle whether to tangle the R code from the input file (like
 #'   \code{\link[utils]{Stangle}})
 #' @param text a character vector as an alternative way to provide the input
-#'   file (\code{text} is written into a temporary file as \code{input}); this
-#'   argument is mainly for test purposes only
+#'   file
 #' @return The compiled document is written into the output file, and the path
-#'   of the output file is returned.
+#'   of the output file is returned, but if the \code{output} path is
+#'   \code{NULL}, the output is returned as a character vector.
 #' @note The name \code{knit} comes from its counterpart \samp{weave} (as in
 #'   Sweave), and the name \code{purl} (as \samp{tangle} in Stangle) comes from
 #'   a knitting method `knit one, purl one'.
-#'   
+#'
 #'   If the input document has child documents, they will also be compiled
 #'   recursively. See \code{\link{knit_child}}.
-#'   
+#'
 #'   The working directory when evaluating R code chunks is the directory of the
 #'   input document, so if the R code involves with external files (like
 #'   \code{read.table()}), it is better to put these files under the same
 #'   directory of the input document so that we can use relative paths.
-#'   
+#'
 #'   The arguments \code{input} and \code{output} do not have to be restricted
-#'   to files; they can be \code{stdin()}/\code{stdout()} or other types of 
+#'   to files; they can be \code{stdin()}/\code{stdout()} or other types of
 #'   connections, but the pattern list to read the input has to be set in
 #'   advance (see \code{\link{pat_rnw}}), and the output hooks should also be
-#'   set (see \code{\link{render_latex}}).
+#'   set (see \code{\link{render_latex}}), otherwise \pkg{knitr} will try to
+#'   guess the patterns and output format.
 #' @export
 #' @references Package homepage: \url{http://yihui.name/knitr/}
 #'
-#' The \pkg{knitr} main manual:
-#' \url{https://github.com/downloads/yihui/knitr/knitr-manual.pdf}
+#'   The \pkg{knitr} main manual:
+#'   \url{https://github.com/downloads/yihui/knitr/knitr-manual.pdf}
 #'
-#' The \pkg{knitr} graphics manual:
-#' \url{https://github.com/downloads/yihui/knitr/knitr-graphics.pdf}
+#'   The \pkg{knitr} graphics manual:
+#'   \url{https://github.com/downloads/yihui/knitr/knitr-graphics.pdf}
 #' @examples library(knitr)
 #' (f = tempfile(fileext = '.Rnw'))
 #' file.copy(system.file('examples', 'knitr-minimal.Rnw', package = 'knitr'),
@@ -84,73 +91,87 @@
 #'
 #' purl(f)  # extract R code only
 knit = function(input, output = NULL, tangle = FALSE, text = NULL) {
-  if (is.character(text)) {
-    input = tempfile(); writeLines(text, con = input)
-  }
-  normal.input = is.character(input)
+
+  in.file = !missing(input) && is.character(input)  # is a file input
   opts_knit$set(tangle = tangle)
-  if (is.null(output)) {
-    output = basename(auto_out_name(input))
+  if (child_mode()) {
+    setwd(opts_knit$get('output.dir')) # always restore original working dir
+    ## in child mode, input path needs to be adjusted
+    if (in.file && !is_abs_path(input))
+      input = file.path(input_dir(), opts_knit$get('child.path'), input)
+  } else opts_knit$set(output.dir = getwd()) # record working directory in 1st run
+
+  ext = 'unknown'
+  if (in.file) {
+    opts_knit$set(input.dir = dirname(input)) # record input dir
+    if (is.null(output)) output = basename(auto_out_name(input))
+    ext = tolower(file_ext(input))
+    options(tikzMetricsDictionary = tikz_dict(input)) # cache tikz dictionary
   }
-  ext = if (normal.input) tolower(file_ext(input)) else 'unknown'
-  apat = opts_knit$get('all.patterns')
+
+  text = if (is.null(text)) readLines(input, warn = FALSE) else {
+    unlist(strsplit(text, '\n', fixed = TRUE)) # make sure each element is one line
+  }
+  if (!length(text)) return() # a trivial case: simply and exit
+
   optc = opts_chunk$get()
   on.exit({opts_chunk$restore(); opts_chunk$set(optc)}, add = TRUE)
-  opat = knit_patterns$get()
+  apat = all_patterns; opat = knit_patterns$get()
   on.exit({knit_patterns$restore(); knit_patterns$set(opat)}, add = TRUE)
   if (length(opat) == 0 || all(sapply(opat, is.null))) {
-    pattern = if (ext == 'md') 'html' else ext
+    # use ext if cannot auto detect pattern
+    if (is.null(pattern <- detect_pattern(text)))
+      pattern = if (ext %in% c('htm', 'rhtm', 'rhtml')) 'html' else {
+        if (ext %in% c('rmd', 'rmarkdown', 'markdown')) 'md' else {
+          if (ext == 'rrst') 'rst' else ext
+        }
+      }
     if (!(pattern %in% names(apat)))
       stop("a pattern list cannot be automatically found for the file extension '",
            ext, "' in built-in pattern lists; ",
            'see ?knit_patterns on how to set up customized patterns')
     knit_patterns$restore()
     knit_patterns$set(apat[[pattern]])
+    opts_knit$set(out.format = switch(pattern, rnw = 'latex', tex = 'latex',
+                                      html = 'html', md = 'markdown', rst = 'rst'))
   }
-  
+
   optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
-  if (child_mode()) {
-    ## in child mode, input path needs to be adjusted
-    if (!is_abs_path(input))
-      input = file.path(input_dir(), opts_knit$get('child.path'), input)
-  }
-  if (normal.input) opts_knit$set(input.dir = dirname(input)) # record working dir
-  
+
   if (is.null(opts_knit$get('out.format'))) {
-    fmt = switch(ext, rnw = 'latex', tex = 'latex', html = 'html', md = 'jekyll',
-                 brew = 'brew', {warning('cannot automatically decide the output format');
-                                 'unknown'})
+    fmt = switch(ext, rnw = 'latex', tex = 'latex', htm = 'html', html = 'html',
+                 md = 'markdown', markdown = 'markdown', brew = 'brew', rst = 'rst',
+                 {warning('cannot automatically decide the output format'); 'unknown'})
     ## set built-in hooks
     opts_knit$set(out.format = fmt)
   }
   ## change output hooks only if they are not set beforehand
   if (identical(knit_hooks$get(names(.default.hooks)), .default.hooks)) {
-    switch(opts_knit$get('out.format'), latex = render_latex(), 
+    switch(opts_knit$get('out.format'), latex = render_latex(),
            sweave = render_sweave(), listings = render_listings(),
-           html = render_html(), jekyll = render_jekyll(), 
-           markdown = render_markdown(), gfm = render_gfm())
+           html = render_html(), jekyll = render_jekyll(),
+           markdown = render_markdown(), rst = render_rst())
     on.exit(knit_hooks$restore(), add = TRUE)
   }
 
   on.exit(chunk_counter(reset = TRUE), add = TRUE) # restore counter
-  ## for tikz graphics (cache the dictionary); turn off fancy quotes
-  oopts = options(tikzMetricsDictionary = tikz_dict(input, normal.input), 
-                  useFancyQuotes = FALSE, digits = 4L, width = 75L, warn = 1L)
+  ## turn off fancy quotes, use smaller digits/width, warn immediately
+  oopts = options(useFancyQuotes = FALSE, digits = 4L, width = 75L, warn = 1L)
   on.exit(options(oopts), add = TRUE)
-  
+
   progress = opts_knit$get('progress')
-  if (normal.input) message(ifelse(progress, '\n\n', ''), 'processing file: ', input)
-  res = process_file(input)
+  if (in.file) message(ifelse(progress, '\n\n', ''), 'processing file: ', input)
+  res = process_file(text)
   unlink('NA')  # temp fix to issue 94
-  cat(res, file = output)
+  cat(res, file = if (is.null(output)) '' else output)
   dep_list$restore()  # empty dependency list
-  
-  if (is.character(output) && file.exists(output)) {
+
+  if (in.file && is.character(output) && file.exists(output)) {
     concord_gen(input, output)  # concordance file
     message('output file: ', normalizePath(output), ifelse(progress, '\n', ''))
   }
-  
-  invisible(output)
+
+  invisible(if (is.null(output)) res else output)
 }
 #' @rdname knit
 #' @param ... arguments passed to \code{\link{knit}}
@@ -159,13 +180,13 @@ purl = function(...) {
   knit(..., tangle = TRUE)
 }
 
-process_file = function(path) {
+process_file = function(text) {
   ocode = knit_code$get()
   on.exit({knit_code$restore(); knit_code$set(ocode)}, add = TRUE)
-  groups = split_file(path)
+  groups = split_file(lines = text)
   n = length(groups); res = character(n)
   tangle = opts_knit$get('tangle')
-  
+
   if (opts_knit$get('progress')) {
     pb = txtProgressBar(0, n, char = '>', style = 3)
     on.exit(close(pb), add = TRUE)
@@ -173,55 +194,54 @@ process_file = function(path) {
   for (i in 1:n) {
     if (opts_knit$get('progress')) {
       setTxtProgressBar(pb, i)
-      if (!tangle) cat('\n')
+      if (!tangle) cat('\n')  # under tangle mode, only show one progress bar
       flush.console()
     }
     group = groups[[i]]
     txt = try((if (tangle) process_tangle else process_group)(group), silent = TRUE)
     if (inherits(txt, 'try-error')) {
       print(group)
-      msg = sprintf('Quitting from lines %s: %s', current_lines(i), txt)
-      (if (interactive()) message else stop)(msg)
-      break
+      stop(sprintf('Quitting from lines %s: %s', current_lines(i), txt))
     }
     res[i] = txt
   }
-  
+
   if (!tangle) res = insert_header(res)  # insert header
   concord_output(n = str_count(res, fixed('\n')) + 1L)  # output line numbers
-  
+
   str_c(c(res, ""), collapse = "\n")
 }
 
 auto_out_name = function(input) {
-  ext = file_ext(input)
-  if (opts_knit$get('tangle')) return(str_replace(input, str_c(ext, '$'), 'R'))
-  if (tolower(ext) == 'rnw') return(str_replace(input, str_c(ext, '$'), 'tex'))
-  if (tolower(ext) %in% c('brew', 'tex', 'html', 'md')) {
+  base = file_path_sans_ext(input)
+  if (opts_knit$get('tangle')) return(str_c(base, '.R'))
+  ext = tolower(file_ext(input))
+  if (ext == 'rnw') return(str_c(base, '.tex'))
+  if (ext %in% c('rmd', 'rmarkdown', 'rhtml', 'rhtm', 'rtex', 'rrst'))
+    return(str_c(base, '.', substring(ext, 2L)))
+  if (ext %in% c('brew', 'tex', 'html', 'md')) {
     if (str_detect(input, '_knit_')) {
       return(str_replace(input, '_knit_', ''))
-    } else {
-      return(str_replace(input, str_c('(\\.', ext, ')$'), '-out\\1'))
-    }
+    } else return(str_c(base, '-out.', ext))
   }
   stop('cannot determine the output filename automatically')
 }
 
 #' Knit a child document
-#' 
+#'
 #' This function is for LaTeX only except when it is used to extract R code from
 #' the document; by default it knits a child document and returns the command to
 #' input the result into the main document. It is designed to be used in the
 #' inline R code and serves as the alternative to the \command{SweaveInput}
 #' command in Sweave.
-#' 
+#'
 #' The LaTeX command used to input the child document (usually \samp{input} or
 #' \samp{include}) is from the package option \code{child.command}
 #' (\code{opts_knit$get('child.command')}).
-#' 
+#'
 #' When we call \code{purl()} to extract R code, the code in the child document
 #' is extracted and saved into an R script.
-#' 
+#'
 #' The path of the child document is relative to the parent document.
 #' @param ... arguments passed to \code{\link{knit}}
 #' @param eval logical: whether to evaluate the child document
@@ -229,7 +249,7 @@ auto_out_name = function(input) {
 #'   \code{source("child-doc.R")}, depending on the argument \code{tangle}
 #'   passed in.
 #' @references \url{http://yihui.name/knitr/demo/child/}
-#' @note This function is not supposed be called directly like 
+#' @note This function is not supposed be called directly like
 #'   \code{\link{knit}()}; instead it must be placed in a parent document to let
 #'   \code{\link{knit}()} call it indirectly.
 #' @export
@@ -250,10 +270,10 @@ knit_child = function(..., eval = TRUE) {
 }
 
 #' Automatically create a report based on an R script and a template
-#' 
+#'
 #' This is a convenience function for small-scale automatic reporting based on
 #' an R script and a template.
-#' 
+#'
 #' The first two lines of the R script can contain the title and author of the
 #' report in comments of the form \samp{## title:} and \samp{## author:}. The
 #' template must have a chunk named \samp{auto-report}, which will be used to
@@ -269,9 +289,9 @@ knit_child = function(..., eval = TRUE) {
 #' \dontrun{stitch(s)}
 #'
 #' ## HTML report
-#' out = stitch(s, system.file('misc', 'knitr-minimal_knit_.html', package = 'knitr'))
+#' out = stitch(s, system.file('misc', 'knitr-template.Rhtml', package = 'knitr'))
 #' if (interactive()) browseURL(out)
-stitch = function(script, 
+stitch = function(script,
                   template = system.file('misc', 'knitr-template.Rnw', package = 'knitr'),
                   output = NULL) {
   lines = readLines(script, warn = FALSE)
