@@ -1,24 +1,25 @@
 ## initial pattern list
-.pat.init = list(chunk.begin = NULL, chunk.end = NULL, chunk.code = NULL,
-                inline.code = NULL, global.options = NULL, input.doc = NULL,
-                ref.chunk = NULL, header.begin = NULL, document.begin = NULL, 
-                ref.label = NULL)
+.pat.init = list(
+  chunk.begin = NULL, chunk.end = NULL, chunk.code = NULL, inline.code = NULL,
+  global.options = NULL, input.doc = NULL, ref.chunk = NULL,
+  header.begin = NULL, document.begin = NULL, ref.label = NULL
+)
 
 #' Patterns to match and extract R code in a document
-#' 
+#'
 #' Patterns are regular expressions and will be used in functions like
 #' \code{\link[base]{grep}} to extract R code and chunk options. The object
 #' \code{knit_patterns} controls the patterns currently used; see the references
 #' and examples for usage.
-#' 
+#'
 #' @references Usage: \url{http://yihui.name/knitr/objects}
-#'   
+#'
 #' Components in \code{knit_patterns}: \url{http://yihui.name/knitr/patterns}
 #' @export
 #' @examples library(knitr)
 #' opat = knit_patterns$get() # old pattern list (to restore later)
 #'
-#' apats = opts_knit$get('all.patterns')  # a list of all built-in patterns
+#' apats = all_patterns  # a list of all built-in patterns
 #' str(apats)
 #' knit_patterns$set(apats[['rnw']]) # set pattern list from apats
 #'
@@ -42,7 +43,7 @@ set_pattern = function(type) {
 }
 
 #' Set regular expressions to read input documents
-#' 
+#'
 #' These are convenience functions to set pre-defined pattern lists (the syntax
 #' to read input documents). The function names are built from corresponding
 #' file extensions, e.g. \code{pat_rnw()} can set the Sweave syntax to read Rnw
@@ -50,21 +51,22 @@ set_pattern = function(type) {
 #' @rdname pat_fun
 #' @return The patterns object \code{\link{knit_patterns}} is modified as a side
 #'   effect.
-#' @export
+#' @export pat_rnw pat_brew pat_tex pat_html pat_md pat_rst
 #' @examples ## see how knit_patterns is modified
 #' knit_patterns$get(); pat_rnw(); knit_patterns$get()
 #'
 #' knit_patterns$restore()  # empty the list
 pat_rnw = function() set_pattern('rnw')
 #' @rdname pat_fun
-#' @export
 pat_brew = function() set_pattern('brew')
 #' @rdname pat_fun
-#' @export
 pat_tex = function() set_pattern('tex')
 #' @rdname pat_fun
-#' @export
 pat_html = function() set_pattern('html')
+#' @rdname pat_fun
+pat_md = function() set_pattern('md')
+#' @rdname pat_fun
+pat_rst = function() set_pattern('rst')
 
 ## helper functions
 
@@ -72,3 +74,14 @@ pat_html = function() set_pattern('html')
 group_pattern = function(pattern) {
   !is.null(pattern) && str_detect(pattern, '\\(.+\\)')
 }
+
+## automatically detect the chunk patterns
+detect_pattern = function(text) {
+  for (p in names(all_patterns)) {
+    for (i in c('chunk.begin', 'inline.code')) {
+      pat = all_patterns[[p]][[i]]
+      if (length(pat) && length(grep(pat, text))) return(p)
+    }
+  }
+  NULL
+}
