@@ -74,7 +74,7 @@ color_def = function(col, variable = 'shadecolor') {
     }
   }
   if (length(x) != 3L) stop('invalid color:', col)
-  x = round(x, 3L)
+  if (is.numeric(x)) x = round(x, 3L)
   sprintf('\\definecolor{%s}{rgb}{%s, %s, %s}', variable, x[1], x[2], x[3])
 }
 
@@ -99,7 +99,7 @@ set_preamble = function(input) {
   if (any(is.na(idx))) return()
   options(tikzDocumentDeclaration = str_sub(txt, idx[, 1L], idx[, 2L]))
   preamble = pure_preamble(str_split(str_sub(txt, idx[, 2L] + 1L), '\n')[[1L]])
-  .knitEnv$tikzPackages = c(preamble, '\n')
+  .knitEnv$tikzPackages = c(.header.sweave.cmd, preamble, '\n')
 }
 ## filter out code chunks from preamble if they exist (they do in LyX/Sweave)
 pure_preamble = function(preamble) {
@@ -114,11 +114,11 @@ pure_preamble = function(preamble) {
 }
 
 #' Specify the parent document of child documents
-#' 
+#'
 #' This function extracts the LaTeX preamble of the parent document to use for
 #' the child document, so that the child document can be compiled as an
 #' individual document.
-#' 
+#'
 #' When the preamble of the parent document also contains code chunks and inline
 #' R code, they will be evaluated as if they were in this child document. For
 #' examples, when \pkg{knitr} hooks or other options are set in the preamble of
@@ -136,9 +136,11 @@ pure_preamble = function(preamble) {
 #' @export
 #' @examples ## can use, e.g. \Sexpr{set_parent('parent_doc.Rnw')} or
 #'
-#' ## <<setup-child, include=FALSE>>=
-#' ## set_parent('parent_doc.Rnw')
-#' ## @@
+#' # <<setup-child, include=FALSE>>=
+#'
+#' # set_parent('parent_doc.Rnw')
+#'
+#' # @@
 set_parent = function(parent) {
   if (child_mode()) return() # quit if in child mode
   opts_knit$set(parent = TRUE)
@@ -148,7 +150,7 @@ set_parent = function(parent) {
 
 ## whether to write results as-is?
 output_asis = function(x, options) {
-  is_blank(x) || options$results %in% c('tex', 'asis')
+  is_blank(x) || options$results == 'asis'
 }
 
 ## path relative to dir of the input file
@@ -190,8 +192,8 @@ is_tikz_dev = function(options) {
   'tikz' %in% options$dev && !options$external
 }
 
-tikz_dict = function(path, normal) {
-  str_c(if (normal) file_path_sans_ext(basename(path)) else 'unknown', '-tikzDictionary')
+tikz_dict = function(path) {
+  str_c(file_path_sans_ext(basename(path)), '-tikzDictionary')
 }
 
 ## compatibility with Sweave and old beta versions of knitr
@@ -206,7 +208,11 @@ fix_options = function(options) {
     }
   }
   if (any(idx <- options$dev == 'eps')) options$dev[idx] = 'postscript'
-  
+  if (options$results == 'tex') {
+    warning("option 'results' was changed from 'tex' to 'asis'")
+    options$results = 'asis'
+  }
+
   ## compatibility with old version of knitr
   fig = options$fig
   if (identical(fig, FALSE)) {
@@ -234,7 +240,7 @@ fix_options = function(options) {
     warning("option 'animate' deprecated; use fig.show=animate please")
     options$fig.show = 'animate'
   }
-  
+
   align = options$align
   if (!is.null(align)) {
     warning("option 'align' deprecated; use fig.align instead")
@@ -250,7 +256,7 @@ fix_options = function(options) {
     warning("option 'height' deprecated; use fig.height instead")
     options$fig.height = height
   }
-  
+
   prefix = options$prefix.string
   if (!is.null(prefix)) {
     warning("option 'prefix.string' deprecated; use fig.path instead")
@@ -261,14 +267,14 @@ fix_options = function(options) {
     warning("option 'prefix.cache' deprecated; use cache.path instead")
     options$cache.path = prefix
   }
-  
+
   ## deal with aliases: a1 is real option; a0 is alias
   if (length(a1 <- opts_knit$get('aliases')) && length(a0 <- names(a1))) {
     for (i in seq_along(a1)) {
       options[[a1[i]]] = options[[a0[i]]]
     }
   }
-  
+
   options
 }
 
@@ -301,7 +307,7 @@ child_mode = function() opts_knit$get('child')
 parent_mode = function() opts_knit$get('parent')
 
 #' Path for figure files
-#' 
+#'
 #' The filename of figure files is the combination of options \code{fig.path}
 #' and \code{label}. This function returns the path of figures for the current
 #' chunk by default.
@@ -320,22 +326,20 @@ fig_path = function(suffix = '', options = opts_current$get()) {
 }
 
 #' The environment in which a code chunk is evaluated
-#' 
+#'
 #' This function makes the environment of a code chunk accessible inside a
 #' chunk.
-#' 
+#'
 #' In some special cases, we need access to the environment of the current
-#' chunk; a typical example is when we use \code{source()} in a cached chunk, we
-#' have to make sure the script is executed in the correct environment (should
-#' not use the default local environment). See references for an example.
+#' chunk, e.g., to make sure the code is executed in the correct environment.
 #' @references \url{http://yihui.name/knitr/demo/cache/}
 #' @export
 knit_env = function() {
   .knitEnv$knit_env
 }
 
-#' Convert Rnw to PDF using knit and texi2pdf
-#' 
+#' Convert Rnw to PDF using knit() and texi2pdf()
+#'
 #' Knit the input Rnw document to a tex document, and compile it using
 #' \code{texi2pdf}.
 #' @inheritParams knit
@@ -343,13 +347,15 @@ knit_env = function() {
 #'   compile the tex document to PDF (by default it uses the default setting of
 #'   \code{\link[tools]{texi2pdf}}, which is often PDFLaTeX); this argument will
 #'   be used to temporarily set the environmental variable \samp{PDFLATEX}
+#' @param ... options to be passed to \code{\link[tools]{texi2pdf}}
 #' @author Ramnath Vaidyanathan and Yihui Xie
 #' @export
 #' @importFrom tools texi2pdf
 #' @seealso \code{\link{knit}}, \code{\link[tools]{texi2pdf}}
 #' @examples ## compile with xelatex
+#'
 #' ## knit2pdf(..., compiler = 'xelatex')
-knit2pdf = function(input, output = NULL, compiler = NULL){
+knit2pdf = function(input, output = NULL, compiler = NULL, ...){
   out = knit(input, output)
   owd = setwd(dirname(out)); on.exit(setwd(owd))
   if (!is.null(compiler)) {
@@ -357,14 +363,32 @@ knit2pdf = function(input, output = NULL, compiler = NULL){
     on.exit(Sys.setenv(PDFLATEX = oc), add = TRUE)
     Sys.setenv(PDFLATEX = compiler)
   }
-  texi2pdf(basename(out), clean = TRUE)
+  texi2pdf(basename(out), ...)
 }
 
+#' Convert markdown to HTML using knit() and markdownToHTML()
+#'
+#' This is a convenience function to knit the input markdown source and call
+#' \code{markdownToHTML()} to convert the result to HTML.
+#' @inheritParams knit
+#' @param ... options passed to \code{\link{knit}}
+#' @export
+#' @seealso \code{\link{knit}}, \code{\link[markdown]{markdownToHTML}}
+#' @examples # a minimal example
+#' writeLines(c('# hello markdown', '``` {r hello-random, echo=TRUE}', 'rnorm(5)', '```'), 'test.Rmd')
+#' knit2html('test.Rmd')
+#' if (interactive()) browseURL('test.html')
+knit2html = function(input, ...){
+  out = knit(input, ...)
+  markdown::markdownToHTML(out, str_c(file_path_sans_ext(out), '.html'))
+}
+
+
 #' Run the code in a specified chunk
-#' 
+#'
 #' We can specify a chunk label and use this function to evaluate the code in
 #' this chunk. It is an alternative to the chunk reference in Sweave.
-#' 
+#'
 #' The difference between this type of chunk reference and the chunk option
 #' \code{ref.label} is that the latter can only be used for a chunk so that it
 #' has exactly the same code as the reference chunk, whereas this function makes
@@ -378,18 +402,18 @@ knit2pdf = function(input, output = NULL, compiler = NULL){
 #'   contains a reference to \samp{chunk3}, then if we run \samp{chunk1}, both
 #'   the code in \samp{chunk2} and \samp{chunk3} will be evaluated.
 #' @export
-#' @examples ## In Sweave we use chunk reference like this
-#' # <<a>>=
-#' # 1+1
-#' # @@
-#' # <<b>>=
-#' # <<a>>
-#' # @@
-#'
-#' ## In knitr, we can use the same, or
-#' # <<b>>=
-#' # run_chunk('a')
-#' # @@
+#' @examples # see http://yihui.name/knitr/demo/reference/
 run_chunk = function(label, envir = parent.frame()) {
   eval(parse(text = knit_code$get(label)), envir = envir)
 }
+
+# Indents a Block
+#  Input
+#     "library(ggplot2)\nqplot(wt, mpg, data = mtcars)"
+#  Output
+#          library(ggplot2)
+#          qplot(wt, mpg, data  = mtcars)
+indent_block = function(block, spaces = '    ') {
+  if (!nzchar(block)) return(spaces)
+  line_prompt(block, spaces, spaces)
+}
